@@ -133,7 +133,8 @@ class ButterflyConnection(telepathy.server.Connection,
 
     def _new_client(self, use_http=False):
         if hasattr(self, '_msn_client') and self._msn_client:
-            self._msn_client.logout()
+            if self._msn_client.state != papyon.event.ClientState.CLOSED:
+                self._msn_client.logout()
             self._msn_client._events_handlers.remove(self)
 
         if use_http:
@@ -230,7 +231,8 @@ class ButterflyConnection(telepathy.server.Connection,
     def Disconnect(self):
         logger.info("Disconnecting")
         self.__disconnect_reason = telepathy.CONNECTION_STATUS_REASON_REQUESTED
-        self._msn_client.logout()
+        if self._msn_client.state != papyon.event.ClientState.CLOSED:
+            self._msn_client.logout()
 
     def GetInterfaces(self):
         # The self._interfaces set is only ever touched in ButterflyConnection.__init__,
@@ -451,6 +453,8 @@ class ButterflyConnection(telepathy.server.Connection,
     # papyon.event.OfflineMessagesEventInterface
     def on_oim_messages_fetched(self, messages):
         for message in messages:
+            if message.sender is None:
+                continue
             # Request butterfly text channel (creation, what happen when it exist)
             sender = message.sender
             logger.info('received offline message from %s : %s' % (sender.account, message.text))
@@ -463,6 +467,17 @@ class ButterflyConnection(telepathy.server.Connection,
             # Notify it of the message
             channel.offline_message_received(message)
 
+    # papyon.event.InviteEventInterface
+    def on_invite_file_transfer(self, session):
+        logger.debug("File transfer invite")
+        handle = ButterflyHandleFactory(self, 'contact', session.peer.account,
+                session.peer.network_id)
+
+        props = self._generate_props(telepathy.CHANNEL_TYPE_FILE_TRANSFER,
+                handle, False)
+        channel = self._channel_manager.create_channel_for_props(props,
+                signal=True, session=session)
+
     def _advertise_disconnected(self):
         self._manager.disconnected(self)
 
